//*****************************************************************************
// Instruction helpers
//-----------------------------------------------------------------------------
//
//

`timescale 1ns / 10 ps

//--------------------------------------------------------------------
// Instruction opcodes
//--------------------------------------------------------------------

`ifndef SMIPS_INST_OPCODES
`define SMIPS_INST_OPCODES

`define NOP   32'b000000_00000_00000_00000_00000_000000
`define LW    32'b100011_?????_?????_?????_?????_??????
`define SW    32'b101011_?????_?????_?????_?????_??????
`define ADDIU 32'b001001_?????_?????_?????_?????_??????
`define SLTI  32'b001010_?????_?????_?????_?????_??????
`define SLTIU 32'b001011_?????_?????_?????_?????_??????
`define ANDI  32'b001100_?????_?????_?????_?????_??????
`define ORI   32'b001101_?????_?????_?????_?????_??????
`define XORI  32'b001110_?????_?????_?????_?????_??????
`define LUI   32'b001111_00000_?????_?????_?????_??????
`define SLL   32'b000000_00000_?????_?????_?????_000000
`define SRL   32'b000000_00000_?????_?????_?????_000010
`define SRA   32'b000000_00000_?????_?????_?????_000011
`define SLLV  32'b000000_?????_?????_?????_00000_000100
`define SRLV  32'b000000_?????_?????_?????_00000_000110
`define SRAV  32'b000000_?????_?????_?????_00000_000111
`define ADDU  32'b000000_?????_?????_?????_00000_100001
`define SUBU  32'b000000_?????_?????_?????_00000_100011
`define AND   32'b000000_?????_?????_?????_00000_100100
`define OR    32'b000000_?????_?????_?????_00000_100101
`define XOR   32'b000000_?????_?????_?????_00000_100110
`define NOR   32'b000000_?????_?????_?????_00000_100111
`define SLT   32'b000000_?????_?????_?????_00000_101010
`define SLTU  32'b000000_?????_?????_?????_00000_101011
`define J     32'b000010_?????_?????_?????_?????_??????
`define JAL   32'b000011_?????_?????_?????_?????_??????
`define JR    32'b000000_?????_00000_00000_00000_001000
`define JALR  32'b000000_?????_00000_?????_00000_001001
`define BEQ   32'b000100_?????_?????_?????_?????_??????
`define BNE   32'b000101_?????_?????_?????_?????_??????
`define BLEZ  32'b000110_?????_00000_?????_?????_??????
`define BGTZ  32'b000111_?????_00000_?????_?????_??????
`define BLTZ  32'b000001_?????_00000_?????_?????_??????
`define BGEZ  32'b000001_?????_00001_?????_?????_??????
`define MFC0  32'b010000_00000_?????_?????_00000_000000
`define MTC0  32'b010000_00100_?????_?????_00000_000000

`endif

//--------------------------------------------------------------------
// Instruction bundle
//--------------------------------------------------------------------

`ifndef SMIPS_INST_FIELDS
`define SMIPS_INST_FIELDS

`define INST_OPCODE    31:26
`define INST_RS        25:21
`define INST_RT        20:16
`define INST_RD        15:11
`define INST_SHAMT     10:6
`define INST_FUNCT     5:0
`define INST_IMM       15:0
`define INST_IMM_SIGN  15
`define INST_TARGET    25:0

`endif

//--------------------------------------------------------------------
// Instruction disassembly
//--------------------------------------------------------------------

module dis(inst, InstructionAddress, Clock);

 input [31:0] inst;
 input [31:0] InstructionAddress;
 input Clock;

  wire [ 4:0] rs       = inst[`INST_RS];
  wire [ 4:0] rt       = inst[`INST_RT];
  wire [ 4:0] rd       = inst[`INST_RD];
  wire [ 4:0] shamt    = inst[`INST_SHAMT];
  wire [15:0] imm      = inst[`INST_IMM];
  wire	      imm_sign = inst[`INST_IMM_SIGN];
  wire [25:0] target   = inst[`INST_TARGET];

  reg [167:0] dasm;
reg [31:0] InstructionAddressDel, InstructionAddressDelDel;
	
	always @(posedge Clock) begin
			InstructionAddressDel <= InstructionAddress;
			InstructionAddressDelDel <= InstructionAddressDel;
	end
	
  always @(posedge Clock)
  begin

    if ( inst === 32'bx )
      $sformat( dasm, "x                    " );
    else
    begin    
    
      casez ( inst )
        `NOP   : $display("PC: %x\tnop                  " , InstructionAddressDelDel);
        `LW    : $display("PC: %x\tlw    r%d, %x(r%d) ",   InstructionAddressDelDel, rt, imm, rs    );
        `SW    : $display("PC: %x\tsw    r%d, %x(r%d) ",   InstructionAddressDelDel, rt, imm, rs    );      
        `ADDIU : $display("PC: %x\taddiu r%d, r%d, %x ",   InstructionAddressDelDel, rt, rs,  imm   );
        `SLTI  : $display("PC: %x\tslti  r%d, r%d, %x ",   InstructionAddressDelDel, rt, rs,  imm   );
        `SLTIU : $display("PC: %x\tsltiu r%d, r%d, %x ",   InstructionAddressDelDel, rt, rs,  imm   );
        `ANDI  : $display("PC: %x\tandi  r%d, r%d, %x ",   InstructionAddressDelDel, rt, rs,  imm   );
        `ORI   : $display("PC: %x\tori   r%d, r%d, %x ",   InstructionAddressDelDel, rt, rs,  imm   );
        `XORI  : $display("PC: %x\txori  r%d, r%d, %x ",   InstructionAddressDelDel, rt, rs,  imm   );
        `LUI   : $display("PC: %x\tlui   r%d, %x      ",   InstructionAddressDelDel, rt, imm        );
        `SLL   : $display("PC: %x\tsll   r%d, r%d, %x   ", InstructionAddressDelDel, rt, rt,  shamt );
        `SRL   : $display("PC: %x\tsrl   r%d, r%d, %x   ", InstructionAddressDelDel, rd, rt,  shamt );
        `SRA   : $display("PC: %x\tsra   r%d, r%d, %x   ", InstructionAddressDelDel, rd, rt,  shamt );
        `SLLV  : $display("PC: %x\tsllv  r%d, r%d, r%d  ", InstructionAddressDelDel, rd, rt,  rs    );
        `SRLV  : $display("PC: %x\tsrlv  r%d, r%d, r%d  ", InstructionAddressDelDel, rd, rt,  rs    );
        `SRAV  : $display("PC: %x\tsrav  r%d, r%d, r%d  ", InstructionAddressDelDel, rd, rt,  rs    );
        `ADDU  : $display("PC: %x\taddu  r%d, r%d, r%d  ", InstructionAddressDelDel, rd, rt,  rs    );
        `SUBU  : $display("PC: %x\tsubu  r%d, r%d, r%d  ", InstructionAddressDelDel, rd, rt,  rs    );
        `AND   : $display("PC: %x\tand   r%d, r%d, r%d  ", InstructionAddressDelDel, rd, rt,  rs    );
        `OR    : $display("PC: %x\tor    r%d, r%d, r%d  ", InstructionAddressDelDel, rd, rt,  rs    );
        `XOR   : $display("PC: %x\txor   r%d, r%d, r%d  ", InstructionAddressDelDel, rd, rt,  rs    );
        `NOR   : $display("PC: %x\tnor   r%d, r%d, r%d  ", InstructionAddressDelDel, rd, rt,  rs    );
        `SLT   : $display("PC: %x\tslt   r%d, r%d, r%d  ", InstructionAddressDelDel, rd, rt,  rs    );
        `SLTU  : $display("PC: %x\tsltu  r%d, r%d, r%d  ", InstructionAddressDelDel, rd, rt,  rs    );
        `J     : $display("PC: %x\tj     %x        ",      InstructionAddressDelDel, target         );
        `JAL   : $display("PC: %x\tjal   %x        ",      InstructionAddressDelDel, target         );
        `JR    : $display("PC: %x\tjr    r%d            ", InstructionAddressDelDel, rs             );
        `JALR  : $display("PC: %x\tjalr  r%d, r%d       ", InstructionAddressDelDel, rd, rs         );
        `BEQ   : $display("PC: %x\tbeq   r%d, r%d, %x ",   InstructionAddressDelDel, rs, rt,  imm   );
        `BNE   : $display("PC: %x\tbne   r%d, r%d, %x ",   InstructionAddressDelDel, rs, rt,  imm   );
        `BLEZ  : $display("PC: %x\tblez  r%d, %x      ",   InstructionAddressDelDel, rs, imm        );
        `BGTZ  : $display("PC: %x\tbgtz  r%d, %x      ",   InstructionAddressDelDel, rs, imm        );
        `BLTZ  : $display("PC: %x\tbltz  r%d, %x      ",   InstructionAddressDelDel, rs, imm        );
        `BGEZ  : $display("PC: %x\tbgez  r%d, %x      ",   InstructionAddressDelDel, rs, imm        );
        `MFC0  : $display("PC: %x\tmfc0  r%d, r%d       ", InstructionAddressDelDel, rt, rd         );
        `MTC0  : $display("PC: %x\tmtc0  r%d, r%d       ", InstructionAddressDelDel, rt, rd         );
        default: $display("undefined instr      " );               
      endcase

    end    

  end  

  reg [40:0] minidasm;
  
endmodule